DEFINE CLASS UndoEngine AS Custom 
* Can't make this a Session class because VFP
* doesn't handle changing data session properly
* for event handlers.

nDataSession = 0
cUndoTable = FORCEPATH("UndoTrail.DBF", SYS(2023))
oUndoList = .null.  && collection of registered object/method pairs

PROCEDURE Init

This.nDataSession = SET("Datasession")
This.SetUpTable()
This.SetUpList()

ENDPROC

PROCEDURE SetUpTable
* Set up the table to store the Undo data

IF NOT FILE(This.cUndoTable)
	CREATE TABLE (This.cUndoTable) ;
		(iID I AUTOINC UNIQUE, cCategory C(20), cKey C(100), mData M, tAdded T, tUndone T)
	INDEX on UPPER(cCategory) TAG cCategory
	INDEX on tAdded TAG tAdded
	INDEX on tUndone TAG tUndone
	USE 
ENDIF

USE (This.cUndoTable) ALIAS __UndoTrail IN 0

RETURN
ENDPROC

PROCEDURE SetUpList

This.oUndoList = CREATEOBJECT("Collection")

RETURN
ENDPROC

PROCEDURE Register(oObject, cMethod, cCategory, cCollectMethod, cRestoreMethod)
* Set up a form or class to add data to the undo trail
* oObject = object registering
* cMethod = method to be registered
* cCollectMethod = method of oObject to call to collect undo data
* cRestoreMethod = method of oObject to call to restore upon undo

LOCAL oUndoData, cKey

cKey = UPPER(ALLTRIM(m.oObject.Name) + "_" + ALLTRIM(m.cMethod))
IF This.oUndoList.GetKey(cKey) = 0
	oUndoData = CREATEOBJECT("Empty")
	ADDPROPERTY(oUndoData, "oObject", m.oObject)
	ADDPROPERTY(oUndoData, "cMethod", m.cMethod)
	ADDPROPERTY(oUndoData, "cCategory", m.cCategory)
	ADDPROPERTY(oUndoData, "cCollectMethod", m.cCollectMethod)
	ADDPROPERTY(oUndoData, "cRestoreMethod", m.cRestoreMethod)

	This.oUndoList.Add(m.oUndoData, m.cKey)
	
	BINDEVENT(m.oObject, m.cMethod, This, "AddToUndo")
ELSE
	This.oUndoList[m.cKey].cCollectMethod= m.cCollectMethod
	This.oUndoList[m.cKey].cRestoreMethod = m.cRestoreMethod
ENDIF 

RETURN
ENDPROC

PROCEDURE AddToUndo
* Add an event to the undo chain. This is a common event sink so that control resides in
* one place.
* Specify parameters in case the bound method has them
LPARAMETERS uParm1, uParm2, uParm3, uParm4, uParm5, uParm6, uParm7, uParm8, uParm9

LOCAL aTrigger[1], cKey, nParms, cMethodToCall, cDataToStore, cParms, nParm
LOCAL nOldSession

nOldSession = SET("Datasession")
SET DATASESSION TO This.nDataSession

nParms = PCOUNT()
IF AEVENTS(aTrigger, 0) <> 0

	cKey = UPPER(ALLTRIM(aTrigger[1].Name) + "_" + ALLTRIM(aTrigger[2]))
	IF This.oUndoList.GetKey(m.cKey) <> 0
		* Gather necessary data
		cMethodToCall = This.oUndoList[m.cKey].cCollectMethod
		IF m.nParms = 0
			cDataToStore = EVALUATE("aTrigger[1] ." + m.cMethodToCall + "()")

		ELSE
		
			cParms = ""
			FOR nParm = 1 TO m.nParms
				cParms = m.cParms + ", uParm" + TRANSFORM(m.nParm)
			ENDFOR
			cParms = SUBSTR(m.cParms, 3)
		
		
			cDataToStore = EVALUATE("aTrigger[1] ." + m.cMethodToCall + "(" + m.cParms + ")")
		ENDIF
		
		* Add a record
		INSERT INTO __UndoTrail  (cCategory, cKey, mData, tAdded) ;
			VALUES (This.oUndoList[m.cKey].cCategory, m.cKey, m.cDataToStore, DATETIME())
	ENDIF 
ENDIF

SET DATASESSION TO (nOldSession)

RETURN
ENDPROC

PROCEDURE Undo
* Undo one action from a specified category
LPARAMETERS cCategory

LOCAL oUndoData, cUndoCall, lReturn

SELECT * ;
	FROM __UndoTrail ;
	WHERE cCategory = m.cCategory ;
		AND EMPTY(tUndone) ;
		AND tAdded = ;
	     	(SELECT MAX(tAdded) FROM __UndoTrail ;
	     	  WHERE EMPTY(tUndone) AND cCategory = m.cCategory) ;
	ORDER BY iID DESC ;
	INTO CURSOR UndoThis

IF _Tally > 0
	oUndoData = This.oUndoList[ ALLTRIM(UndoThis.cKey)]
	cUndoCall = ALLTRIM(oUndoData.cRestoreMethod) + "([" + ALLTRIM(UndoThis.mData) + "])"
	lReturn = oUndoData.oObject.&cUndoCall

	IF m.lReturn
		UPDATE __UndoTrail ;
			SET tUndone = DATETIME() ;
			WHERE iID = UndoThis.iID
	ENDIF
ENDIF

USE IN UndoThis

RETURN m.lReturn
ENDPROC

			
PROCEDURE Destroy

* Close table
USE IN SELECT("__UndoTrail")

This.CleanUp()
ENDPROC

PROCEDURE CleanUp

This.oUndoList.Remove(-1)

RETURN
ENDPROC

ENDDEFINE
