DEFINE CLASS GetUserColors AS Custom

oUserColors = .null. && Collection for user colors
nOldProc = 0 && needed for event binding

PROCEDURE Init

This.oUserColors = CREATEOBJECT("Collection")

DECLARE INTEGER GetSysColor IN WIN32API INTEGER nElement
This.ReadUserColors()

This.BindColorChanges()

ENDPROC

PROCEDURE Destroy

USE IN (SELECT("ColorNames"))
This.oUserColors = .null.

ENDPROC

PROCEDURE ReadUserColors
* Reads the colors from the user's current theme/scheme

LOCAL nColor, oColor 

This.GetColorNames()
* Clear collection
This.oUserColors.Remove(-1)

SELECT ColorNames
SCAN
	nColor = GetSysColor(ColorNames.nIndex)
	oColor = CREATEOBJECT("Empty")
	ADDPROPERTY(oColor, "Color", nColor)
	ADDPROPERTY(oColor, "Name", ColorNames.cName)
	
	This.oUserColors.Add(oColor, ALLTRIM(oColor.Name))
ENDSCAN

RETURN
ENDPROC

PROCEDURE GetColorNames
* Populate a cursor with the names and indexes of Windows colors
CREATE CURSOR ColorNames (cName C(30), nIndex N(2))

INSERT INTO ColorNames VALUES ("SCROLLBAR", 0)
INSERT INTO ColorNames VALUES ("DESKTOP ", 1)
INSERT INTO ColorNames VALUES ("ACTIVECAPTION", 2)
INSERT INTO ColorNames VALUES ("INACTIVECAPTION",  3)
INSERT INTO ColorNames VALUES ("MENU",  4)
INSERT INTO ColorNames VALUES ("WINDOW",  5)
INSERT INTO ColorNames VALUES ("WINDOWFRAME",  6)
INSERT INTO ColorNames VALUES ("MENUTEXT",  7)
INSERT INTO ColorNames VALUES ("WINDOWTEXT",  8)
INSERT INTO ColorNames VALUES ("CAPTIONTEXT",  9)
INSERT INTO ColorNames VALUES ("ACTIVEBORDER",  10)
INSERT INTO ColorNames VALUES ("INACTIVEBORDER",  11)
INSERT INTO ColorNames VALUES ("APPWORKSPACE",  12)
INSERT INTO ColorNames VALUES ("HIGHLIGHT",  13)
INSERT INTO ColorNames VALUES ("HIGHLIGHTTEXT",  14)
INSERT INTO ColorNames VALUES ("3DFACE",  15)
INSERT INTO ColorNames VALUES ("3DSHADOW",  16)
INSERT INTO ColorNames VALUES ("GRAYTEXT",  17)
INSERT INTO ColorNames VALUES ("BTNTEXT",  18)
INSERT INTO ColorNames VALUES ("INACTIVECAPTIONTEXT",  19)
INSERT INTO ColorNames VALUES ("3DHIGHLIGHT",  20)
INSERT INTO ColorNames VALUES ("3DDKSHADOW",  21)
INSERT INTO ColorNames VALUES ("3DLIGHT",  22)
INSERT INTO ColorNames VALUES ("INFOTEXT",  23)
INSERT INTO ColorNames VALUES ("INFOBK",  24)
INSERT INTO ColorNames VALUES ("HOTLIGHT", 26)
INSERT INTO ColorNames VALUES ("GRADIENTACTIVECAPTION", 27)
INSERT INTO ColorNames VALUES ("GRADIENTINACTIVECAPTION", 28)
INSERT INTO ColorNames VALUES ("MENUHILIGHT", 29)
INSERT INTO ColorNames VALUES ("MENUBAR", 30)
RETURN
ENDPROC

PROCEDURE BindColorChanges
* Bind the info here to changes in the user's theme/scheme

* Prepare for binding
DECLARE integer CallWindowProc IN WIN32API ;
	integer lpPrevWndFunc, ;
	integer hWnd,integer Msg,;
	integer wParam,;
	integer lParam

DECLARE integer GetWindowLong IN WIN32API ;
	integer hWnd, ;
	integer nIndex

THIS.nOldProc=GetWindowLong(_SCREEN.HWnd, -4) && GWL_WNDPROC

BINDEVENT(_SCREEN.hWnd, 0x031A, THIS, "HandleThemeChange") && WM_THEMECHANGED
BINDEVENT(_SCREEN.hWnd, 0x0015, THIS, "HandleThemeChange") &&WM_SYSCOLORCHANGE		

RETURN
ENDPROC

PROCEDURE HandleThemeChange
* Respond to user's change of theme/scheme

LPARAMETERS hWnd as Integer, Msg as Integer, wParam as Integer, lParam as Integer

LOCAL lResult
lResult=0

* Note: for WM_THEMECHANGED, MSDN indicates the wParam and lParam 
* are reserved so can't use them.

lResult=CallWindowProc(this.nOldProc,hWnd,msg,wParam,lParam)

This.ReadUserColors()

RETURN lResult
ENDPROC 

PROCEDURE GetAColor(cColor)

LOCAL nReturn

* Does it exist?
IF This.oUserColors.GetKey(m.cColor) = 0
	nReturn = -1
ELSE 
	nReturn = This.oUserColors[m.cColor].Color
ENDIF 

RETURN nReturn
ENDPROC

ENDDEFINE
