* Start library application
PUBLIC glIsDemo && to bump fonts
glIsDemo = .T.

PRIVATE goApp
goApp = .null.

SET PATH TO Data;Bitmaps

* Start main form (and main menu)
DO FORM MainForm

* Splash screen to cover start-up
DO FORM Splash

* Create app object.
* Main toolbar will be set up as part of this object
* On shutdown is part of app object
goApp = CREATEOBJECT("AppObject", MainForm)

* Clear splash screen
Splash.Release()

* Event loop
READ EVENTS
 
IF VARTYPE(MainForm) = "O"
	MainForm.Release()
ENDIF 

RETURN

DEFINE CLASS AppObject AS Custom

oActiveForm = .null.
oMainToolbar = .null.
nDailyFine = .05
nFormCount = 0
oToolbars = .null.
lLargeToolbars = .F.
lSearchOpen = .F.
oSearchForm = .null.
oUndoEngine = .null.

PROCEDURE Init(oMainForm)

This.SetSettings()

* This.oUndoEngine = NEWOBJECT("UndoEngine", "UndoEngine.PRG")

This.oActiveForm = oMainForm
This.BindForm(This.oActiveForm)

This.SetupToolbars()

RETURN
ENDPROC 

PROCEDURE Destroy
* Clean up
This.oActiveForm = .null.
This.oMainToolbar = .null.
This.oSearchForm = .null.
This.oUndoEngine = .null.

RETURN
ENDPROC


PROCEDURE BindForm(oForm)
* Bind a form's events as needed

BINDEVENT(oForm, "Activate", This, "SetActiveForm", 1)
BINDEVENT(oForm, "Deactivate", This, "ClearActiveForm")
IF PEMSTATUS(oForm, "GotBarCode",5)
	BINDEVENT(oForm, "GotBarCode", This, "ProcessBarCode")
ENDIF

RETURN
ENDPROC

PROCEDURE SetActiveForm

IF PEMSTATUS(_Screen,"ActiveForm", 5) AND NOT ISNULL(_Screen.ActiveForm)
	This.oActiveForm = _Screen.ActiveForm
ENDIF 
IF NOT ISNULL(This.oActiveForm)
	nCount = AMEMBERS(aofm, This.oActiveForm, 1)
ENDIF 

RETURN
ENDPROC

PROCEDURE ClearActiveForm

This.oActiveForm = .null.

RETURN
ENDPROC

PROCEDURE SetSettings

SET SAFETY OFF
SET DELETED ON
SET EXCLUSIVE OFF
SET CENTURY ON
SET TALK OFF
SET CONSOLE OFF

RETURN
ENDPROC 

PROCEDURE SetupToolbars
* Start base toolbars

This.oMainToolbar = NEWOBJECT("tbrMain","LibraryToolbars")
This.oMainToolbar.Show()

This.oToolbars = CREATEOBJECT("Collection")

BINDEVENT(This, "lLargeToolbars", This, "ChangeToolbarSize", 1)

RETURN
ENDPROC

PROCEDURE ProcessBarCode(cBarCode)

LOCAL cCode
* Determine what type of code it is and act accordingly
* For this example app, we'll specify that
* codes for books are 12 characters and 
* codes for members are 10.

cCode = ALLTRIM(m.cBarCode)
* Strip off boundary characters
cCode = SUBSTR(m.cCode, 2)

DO CASE 
CASE LEN(m.cCode) = 12
	* It's a book
	This.CheckBookStatus(m.cCode)
	
CASE LEN(m.cCode) = 10
	* It's a member
	This.ActivateCheckoutWithMember(m.cCode)

OTHERWISE
	* Invalid code--do something
ENDCASE

RETURN
ENDPROC

PROCEDURE ActivateCheckoutWithMember(cMemberCode)
* If form is active, bring to front; otherwise start it, 
* passing specified parameters.

LOCAL oForm, lFound

lFound = .F.
FOR EACH oForm IN _SCREEN.Forms
	IF UPPER(oForm.Name) = "FRMCHECKOUT"
		IF oForm.NeedsMember()
			oForm.GetMember(m.cMemberCode)
			oForm.Activate()
			oForm.txtBookBarCode.SetFocus()
			lFound = .T.
			EXIT 
		ENDIF
	ENDIF
ENDFOR

IF NOT lFound
	DO FORM CheckOut WITH m.cMemberCode
ENDIF 

RETURN 
ENDPROC 

PROCEDURE CheckBookStatus(m.cBookCode)
* Figure out whether this book is in or out
* and activate the appropriate form.

LOCAL oBiz, oDE, lCheckedOut, lFound, oForm

oDE = NEWOBJECT("DEBookLookup", "DataDEs")
oBiz = NEWOBJECT("BizBookLookup","BizObjs")
oBiz.oDE = m.oDE
oBiz.oDE.OpenTables()

lCheckedOut = oBiz.GetBookStatus(m.cBookCode)

IF lCheckedOut
	* Check it in
	FOR EACH oForm IN _SCREEN.Forms
		IF UPPER(oForm.Name) = "FRMCHECKIN"
			oForm.AddCheckIn(m.cBookCode)
			oForm.Refresh()
			lFound = .T.
			EXIT 
		ENDIF
	ENDFOR
	
	IF NOT lFound
		DO FORM CheckIn WITH m.cBookCode
	ENDIF
	
ELSE
	* Add to checkout
	FOR EACH oForm IN _SCREEN.Forms
		IF UPPER(oForm.Name) = "FRMCHECKOUT"
			oForm.AddCheckOut(m.cBookCode)
			oForm.Refresh()
			lFound = .T.
			EXIT
		ENDIF
	ENDFOR
	
	IF NOT lFound
		DO FORM CheckOut WITH m.cBookCode, .T.
	ENDIF
ENDIF


RETURN
ENDPROC
		
PROCEDURE AddForm

This.nFormCount = This.nFormCount + 1 

RETURN
ENDPROC

PROCEDURE RemoveForm

This.nFormCount = This.nFormCount - 1 

RETURN
ENDPROC

PROCEDURE AddToolbar(cToolbarClass, cToolbarClassLib, cToolbarName, oForm)

LOCAL oToolbar

TRY
	IF This.oToolbars.GetKey(m.cToolbarName) = 0
		oToolbar = NEWOBJECT(m.cToolbarClass, m.cToolbarClassLib,"",m.oForm)
		oTBObject = CREATEOBJECT("Empty")
		ADDPROPERTY(oTBObject, "oToolbar", oToolbar)
		ADDPROPERTY(oTBObject, "nUsedCount", 1)
		This.oToolbars.Add(oTBObject, m.cToolbarName)
		oToolbar.Show()
	ELSE
		* Increment the counter
		This.oToolbars[m.cToolbarName].nUsedCount = This.oToolbars[m.cToolbarName].nUsedCount + 1 
	ENDIF 
CATCH
ENDTRY

RETURN
ENDPROC

PROCEDURE RemoveToolbar(cToolbarName)
* Remove the specified toolbar if no form is using it.

IF This.oToolbars.GetKey(m.cToolbarName) <> 0
	oTBObject = This.oToolbars[m.cToolbarName]
	IF oTBObject.nUsedCount = 1
		oTBObject.oToolbar.Release()
		This.oToolbars.Remove(This.oToolbars.GetKey(m.cToolbarName))
	ELSE
		oTBObject.nUsedCount = oTBObject.nUsedCount - 1 
	ENDIF 
ENDIF 

RETURN
ENDPROC

PROCEDURE ChangeToolbarSize

LOCAL cNewSize, oToolbar

IF This.lLargeToolbars
	cNewSize = "Large"
ELSE
	cNewSize = "Small"
ENDIF

IF PEMSTATUS(This.oMainToolbar, "ChangeButtonSize", 5)
	This.oMainToolbar.ChangeButtonSize(m.cNewSize)
ENDIF

FOR EACH oToolbar IN This.oToolbars
	IF PEMSTATUS(oToolbar.oToolbar, "ChangeButtonSize", 5)
		oToolbar.oToolbar.ChangeButtonSize(m.cNewSize)
	ENDIF
ENDFOR

RETURN
ENDPROC

PROCEDURE ToggleSearch
* STart or stop the search form

IF This.lSearchOpen
	This.oSearchForm.Release()
	SET MARK OF BAR 1 OF REference TO .F.
	This.oMainToolbar.chkCatalogSearch.Value = .f.
	This.lSearchOpen = .F.
ELSE 
	DO FORM SearchCatalog with MainForm
	This.oSearchForm = This.oActiveForm
	SET MARK OF BAR 1 OF REference TO .T.
	This.oMainToolbar.chkCatalogSearch.Value = .t.
	This.lSearchOpen = .T.
ENDIF



ENDDEFINE

